# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.magic_cloud_networking import (
    resource_get_params,
    resource_list_params,
    resource_export_params,
    resource_policy_preview_params,
)
from ...types.magic_cloud_networking.resource_get_response import ResourceGetResponse
from ...types.magic_cloud_networking.resource_list_response import ResourceListResponse
from ...types.magic_cloud_networking.resource_policy_preview_response import ResourcePolicyPreviewResponse

__all__ = ["ResourcesResource", "AsyncResourcesResource"]


class ResourcesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ResourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ResourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ResourcesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cloudflare: bool | Omit = omit,
        desc: bool | Omit = omit,
        managed: bool | Omit = omit,
        order_by: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        provider_id: str | Omit = omit,
        region: str | Omit = omit,
        resource_group: str | Omit = omit,
        resource_id: SequenceNotStr[str] | Omit = omit,
        resource_type: List[
            Literal[
                "aws_customer_gateway",
                "aws_egress_only_internet_gateway",
                "aws_internet_gateway",
                "aws_instance",
                "aws_network_interface",
                "aws_route",
                "aws_route_table",
                "aws_route_table_association",
                "aws_subnet",
                "aws_vpc",
                "aws_vpc_ipv4_cidr_block_association",
                "aws_vpn_connection",
                "aws_vpn_connection_route",
                "aws_vpn_gateway",
                "aws_security_group",
                "aws_vpc_security_group_ingress_rule",
                "aws_vpc_security_group_egress_rule",
                "aws_ec2_managed_prefix_list",
                "aws_ec2_transit_gateway",
                "aws_ec2_transit_gateway_prefix_list_reference",
                "aws_ec2_transit_gateway_vpc_attachment",
                "azurerm_application_security_group",
                "azurerm_lb",
                "azurerm_lb_backend_address_pool",
                "azurerm_lb_nat_pool",
                "azurerm_lb_nat_rule",
                "azurerm_lb_rule",
                "azurerm_local_network_gateway",
                "azurerm_network_interface",
                "azurerm_network_interface_application_security_group_association",
                "azurerm_network_interface_backend_address_pool_association",
                "azurerm_network_interface_security_group_association",
                "azurerm_network_security_group",
                "azurerm_public_ip",
                "azurerm_route",
                "azurerm_route_table",
                "azurerm_subnet",
                "azurerm_subnet_route_table_association",
                "azurerm_virtual_machine",
                "azurerm_virtual_network_gateway_connection",
                "azurerm_virtual_network",
                "azurerm_virtual_network_gateway",
                "google_compute_network",
                "google_compute_subnetwork",
                "google_compute_vpn_gateway",
                "google_compute_vpn_tunnel",
                "google_compute_route",
                "google_compute_address",
                "google_compute_global_address",
                "google_compute_router",
                "google_compute_interconnect_attachment",
                "google_compute_ha_vpn_gateway",
                "google_compute_forwarding_rule",
                "google_compute_network_firewall_policy",
                "google_compute_network_firewall_policy_rule",
                "cloudflare_static_route",
                "cloudflare_ipsec_tunnel",
            ]
        ]
        | Omit = omit,
        search: SequenceNotStr[str] | Omit = omit,
        v2: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[ResourceListResponse]:
        """
        List resources in the Resource Catalog (Closed Beta).

        Args:
          order_by: One of ["id", "resource_type", "region"].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/resources", account_id=account_id),
            page=SyncV4PagePaginationArray[ResourceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cloudflare": cloudflare,
                        "desc": desc,
                        "managed": managed,
                        "order_by": order_by,
                        "page": page,
                        "per_page": per_page,
                        "provider_id": provider_id,
                        "region": region,
                        "resource_group": resource_group,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "search": search,
                        "v2": v2,
                    },
                    resource_list_params.ResourceListParams,
                ),
            ),
            model=ResourceListResponse,
        )

    def export(
        self,
        *,
        account_id: str,
        desc: bool | Omit = omit,
        order_by: str | Omit = omit,
        provider_id: str | Omit = omit,
        region: str | Omit = omit,
        resource_group: str | Omit = omit,
        resource_id: SequenceNotStr[str] | Omit = omit,
        resource_type: List[
            Literal[
                "aws_customer_gateway",
                "aws_egress_only_internet_gateway",
                "aws_internet_gateway",
                "aws_instance",
                "aws_network_interface",
                "aws_route",
                "aws_route_table",
                "aws_route_table_association",
                "aws_subnet",
                "aws_vpc",
                "aws_vpc_ipv4_cidr_block_association",
                "aws_vpn_connection",
                "aws_vpn_connection_route",
                "aws_vpn_gateway",
                "aws_security_group",
                "aws_vpc_security_group_ingress_rule",
                "aws_vpc_security_group_egress_rule",
                "aws_ec2_managed_prefix_list",
                "aws_ec2_transit_gateway",
                "aws_ec2_transit_gateway_prefix_list_reference",
                "aws_ec2_transit_gateway_vpc_attachment",
                "azurerm_application_security_group",
                "azurerm_lb",
                "azurerm_lb_backend_address_pool",
                "azurerm_lb_nat_pool",
                "azurerm_lb_nat_rule",
                "azurerm_lb_rule",
                "azurerm_local_network_gateway",
                "azurerm_network_interface",
                "azurerm_network_interface_application_security_group_association",
                "azurerm_network_interface_backend_address_pool_association",
                "azurerm_network_interface_security_group_association",
                "azurerm_network_security_group",
                "azurerm_public_ip",
                "azurerm_route",
                "azurerm_route_table",
                "azurerm_subnet",
                "azurerm_subnet_route_table_association",
                "azurerm_virtual_machine",
                "azurerm_virtual_network_gateway_connection",
                "azurerm_virtual_network",
                "azurerm_virtual_network_gateway",
                "google_compute_network",
                "google_compute_subnetwork",
                "google_compute_vpn_gateway",
                "google_compute_vpn_tunnel",
                "google_compute_route",
                "google_compute_address",
                "google_compute_global_address",
                "google_compute_router",
                "google_compute_interconnect_attachment",
                "google_compute_ha_vpn_gateway",
                "google_compute_forwarding_rule",
                "google_compute_network_firewall_policy",
                "google_compute_network_firewall_policy_rule",
                "cloudflare_static_route",
                "cloudflare_ipsec_tunnel",
            ]
        ]
        | Omit = omit,
        search: SequenceNotStr[str] | Omit = omit,
        v2: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Export resources in the Resource Catalog as a JSON file (Closed Beta).

        Args:
          order_by: One of ["id", "resource_type", "region"].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._get(
            path_template("/accounts/{account_id}/magic/cloud/resources/export", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "desc": desc,
                        "order_by": order_by,
                        "provider_id": provider_id,
                        "region": region,
                        "resource_group": resource_group,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "search": search,
                        "v2": v2,
                    },
                    resource_export_params.ResourceExportParams,
                ),
            ),
            cast_to=BinaryAPIResponse,
        )

    def get(
        self,
        resource_id: str,
        *,
        account_id: str,
        v2: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResourceGetResponse:
        """
        Read an resource from the Resource Catalog (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/cloud/resources/{resource_id}",
                account_id=account_id,
                resource_id=resource_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"v2": v2}, resource_get_params.ResourceGetParams),
                post_parser=ResultWrapper[ResourceGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ResourceGetResponse], ResultWrapper[ResourceGetResponse]),
        )

    def policy_preview(
        self,
        *,
        account_id: str,
        policy: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Preview Rego query result against the latest resource catalog (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/magic/cloud/resources/policy-preview", account_id=account_id),
            body=maybe_transform({"policy": policy}, resource_policy_preview_params.ResourcePolicyPreviewParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ResourcePolicyPreviewResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class AsyncResourcesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncResourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncResourcesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cloudflare: bool | Omit = omit,
        desc: bool | Omit = omit,
        managed: bool | Omit = omit,
        order_by: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        provider_id: str | Omit = omit,
        region: str | Omit = omit,
        resource_group: str | Omit = omit,
        resource_id: SequenceNotStr[str] | Omit = omit,
        resource_type: List[
            Literal[
                "aws_customer_gateway",
                "aws_egress_only_internet_gateway",
                "aws_internet_gateway",
                "aws_instance",
                "aws_network_interface",
                "aws_route",
                "aws_route_table",
                "aws_route_table_association",
                "aws_subnet",
                "aws_vpc",
                "aws_vpc_ipv4_cidr_block_association",
                "aws_vpn_connection",
                "aws_vpn_connection_route",
                "aws_vpn_gateway",
                "aws_security_group",
                "aws_vpc_security_group_ingress_rule",
                "aws_vpc_security_group_egress_rule",
                "aws_ec2_managed_prefix_list",
                "aws_ec2_transit_gateway",
                "aws_ec2_transit_gateway_prefix_list_reference",
                "aws_ec2_transit_gateway_vpc_attachment",
                "azurerm_application_security_group",
                "azurerm_lb",
                "azurerm_lb_backend_address_pool",
                "azurerm_lb_nat_pool",
                "azurerm_lb_nat_rule",
                "azurerm_lb_rule",
                "azurerm_local_network_gateway",
                "azurerm_network_interface",
                "azurerm_network_interface_application_security_group_association",
                "azurerm_network_interface_backend_address_pool_association",
                "azurerm_network_interface_security_group_association",
                "azurerm_network_security_group",
                "azurerm_public_ip",
                "azurerm_route",
                "azurerm_route_table",
                "azurerm_subnet",
                "azurerm_subnet_route_table_association",
                "azurerm_virtual_machine",
                "azurerm_virtual_network_gateway_connection",
                "azurerm_virtual_network",
                "azurerm_virtual_network_gateway",
                "google_compute_network",
                "google_compute_subnetwork",
                "google_compute_vpn_gateway",
                "google_compute_vpn_tunnel",
                "google_compute_route",
                "google_compute_address",
                "google_compute_global_address",
                "google_compute_router",
                "google_compute_interconnect_attachment",
                "google_compute_ha_vpn_gateway",
                "google_compute_forwarding_rule",
                "google_compute_network_firewall_policy",
                "google_compute_network_firewall_policy_rule",
                "cloudflare_static_route",
                "cloudflare_ipsec_tunnel",
            ]
        ]
        | Omit = omit,
        search: SequenceNotStr[str] | Omit = omit,
        v2: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ResourceListResponse, AsyncV4PagePaginationArray[ResourceListResponse]]:
        """
        List resources in the Resource Catalog (Closed Beta).

        Args:
          order_by: One of ["id", "resource_type", "region"].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/resources", account_id=account_id),
            page=AsyncV4PagePaginationArray[ResourceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cloudflare": cloudflare,
                        "desc": desc,
                        "managed": managed,
                        "order_by": order_by,
                        "page": page,
                        "per_page": per_page,
                        "provider_id": provider_id,
                        "region": region,
                        "resource_group": resource_group,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "search": search,
                        "v2": v2,
                    },
                    resource_list_params.ResourceListParams,
                ),
            ),
            model=ResourceListResponse,
        )

    async def export(
        self,
        *,
        account_id: str,
        desc: bool | Omit = omit,
        order_by: str | Omit = omit,
        provider_id: str | Omit = omit,
        region: str | Omit = omit,
        resource_group: str | Omit = omit,
        resource_id: SequenceNotStr[str] | Omit = omit,
        resource_type: List[
            Literal[
                "aws_customer_gateway",
                "aws_egress_only_internet_gateway",
                "aws_internet_gateway",
                "aws_instance",
                "aws_network_interface",
                "aws_route",
                "aws_route_table",
                "aws_route_table_association",
                "aws_subnet",
                "aws_vpc",
                "aws_vpc_ipv4_cidr_block_association",
                "aws_vpn_connection",
                "aws_vpn_connection_route",
                "aws_vpn_gateway",
                "aws_security_group",
                "aws_vpc_security_group_ingress_rule",
                "aws_vpc_security_group_egress_rule",
                "aws_ec2_managed_prefix_list",
                "aws_ec2_transit_gateway",
                "aws_ec2_transit_gateway_prefix_list_reference",
                "aws_ec2_transit_gateway_vpc_attachment",
                "azurerm_application_security_group",
                "azurerm_lb",
                "azurerm_lb_backend_address_pool",
                "azurerm_lb_nat_pool",
                "azurerm_lb_nat_rule",
                "azurerm_lb_rule",
                "azurerm_local_network_gateway",
                "azurerm_network_interface",
                "azurerm_network_interface_application_security_group_association",
                "azurerm_network_interface_backend_address_pool_association",
                "azurerm_network_interface_security_group_association",
                "azurerm_network_security_group",
                "azurerm_public_ip",
                "azurerm_route",
                "azurerm_route_table",
                "azurerm_subnet",
                "azurerm_subnet_route_table_association",
                "azurerm_virtual_machine",
                "azurerm_virtual_network_gateway_connection",
                "azurerm_virtual_network",
                "azurerm_virtual_network_gateway",
                "google_compute_network",
                "google_compute_subnetwork",
                "google_compute_vpn_gateway",
                "google_compute_vpn_tunnel",
                "google_compute_route",
                "google_compute_address",
                "google_compute_global_address",
                "google_compute_router",
                "google_compute_interconnect_attachment",
                "google_compute_ha_vpn_gateway",
                "google_compute_forwarding_rule",
                "google_compute_network_firewall_policy",
                "google_compute_network_firewall_policy_rule",
                "cloudflare_static_route",
                "cloudflare_ipsec_tunnel",
            ]
        ]
        | Omit = omit,
        search: SequenceNotStr[str] | Omit = omit,
        v2: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Export resources in the Resource Catalog as a JSON file (Closed Beta).

        Args:
          order_by: One of ["id", "resource_type", "region"].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._get(
            path_template("/accounts/{account_id}/magic/cloud/resources/export", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "desc": desc,
                        "order_by": order_by,
                        "provider_id": provider_id,
                        "region": region,
                        "resource_group": resource_group,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "search": search,
                        "v2": v2,
                    },
                    resource_export_params.ResourceExportParams,
                ),
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def get(
        self,
        resource_id: str,
        *,
        account_id: str,
        v2: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResourceGetResponse:
        """
        Read an resource from the Resource Catalog (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/cloud/resources/{resource_id}",
                account_id=account_id,
                resource_id=resource_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"v2": v2}, resource_get_params.ResourceGetParams),
                post_parser=ResultWrapper[ResourceGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ResourceGetResponse], ResultWrapper[ResourceGetResponse]),
        )

    async def policy_preview(
        self,
        *,
        account_id: str,
        policy: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Preview Rego query result against the latest resource catalog (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/magic/cloud/resources/policy-preview", account_id=account_id),
            body=await async_maybe_transform(
                {"policy": policy}, resource_policy_preview_params.ResourcePolicyPreviewParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ResourcePolicyPreviewResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class ResourcesResourceWithRawResponse:
    def __init__(self, resources: ResourcesResource) -> None:
        self._resources = resources

        self.list = to_raw_response_wrapper(
            resources.list,
        )
        self.export = to_custom_raw_response_wrapper(
            resources.export,
            BinaryAPIResponse,
        )
        self.get = to_raw_response_wrapper(
            resources.get,
        )
        self.policy_preview = to_raw_response_wrapper(
            resources.policy_preview,
        )


class AsyncResourcesResourceWithRawResponse:
    def __init__(self, resources: AsyncResourcesResource) -> None:
        self._resources = resources

        self.list = async_to_raw_response_wrapper(
            resources.list,
        )
        self.export = async_to_custom_raw_response_wrapper(
            resources.export,
            AsyncBinaryAPIResponse,
        )
        self.get = async_to_raw_response_wrapper(
            resources.get,
        )
        self.policy_preview = async_to_raw_response_wrapper(
            resources.policy_preview,
        )


class ResourcesResourceWithStreamingResponse:
    def __init__(self, resources: ResourcesResource) -> None:
        self._resources = resources

        self.list = to_streamed_response_wrapper(
            resources.list,
        )
        self.export = to_custom_streamed_response_wrapper(
            resources.export,
            StreamedBinaryAPIResponse,
        )
        self.get = to_streamed_response_wrapper(
            resources.get,
        )
        self.policy_preview = to_streamed_response_wrapper(
            resources.policy_preview,
        )


class AsyncResourcesResourceWithStreamingResponse:
    def __init__(self, resources: AsyncResourcesResource) -> None:
        self._resources = resources

        self.list = async_to_streamed_response_wrapper(
            resources.list,
        )
        self.export = async_to_custom_streamed_response_wrapper(
            resources.export,
            AsyncStreamedBinaryAPIResponse,
        )
        self.get = async_to_streamed_response_wrapper(
            resources.get,
        )
        self.policy_preview = async_to_streamed_response_wrapper(
            resources.policy_preview,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["MetadataIndexDeleteParams"]


class MetadataIndexDeleteParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    property_name: Required[Annotated[str, PropertyInfo(alias="propertyName")]]
    """Specifies the metadata property for which the index must be deleted."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template
from ...._compat import cached_property
from .operations import (
    OperationsResource,
    AsyncOperationsResource,
    OperationsResourceWithRawResponse,
    AsyncOperationsResourceWithRawResponse,
    OperationsResourceWithStreamingResponse,
    AsyncOperationsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.api_gateway.discovery_get_response import DiscoveryGetResponse

__all__ = ["DiscoveryResource", "AsyncDiscoveryResource"]


class DiscoveryResource(SyncAPIResource):
    @cached_property
    def operations(self) -> OperationsResource:
        return OperationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> DiscoveryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DiscoveryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DiscoveryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DiscoveryResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DiscoveryGetResponse:
        """
        Retrieve the most up to date view of discovered operations, rendered as OpenAPI
        schemas

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/api_gateway/discovery", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DiscoveryGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DiscoveryGetResponse], ResultWrapper[DiscoveryGetResponse]),
        )


class AsyncDiscoveryResource(AsyncAPIResource):
    @cached_property
    def operations(self) -> AsyncOperationsResource:
        return AsyncOperationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDiscoveryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDiscoveryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDiscoveryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDiscoveryResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DiscoveryGetResponse:
        """
        Retrieve the most up to date view of discovered operations, rendered as OpenAPI
        schemas

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/api_gateway/discovery", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DiscoveryGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DiscoveryGetResponse], ResultWrapper[DiscoveryGetResponse]),
        )


class DiscoveryResourceWithRawResponse:
    def __init__(self, discovery: DiscoveryResource) -> None:
        self._discovery = discovery

        self.get = to_raw_response_wrapper(
            discovery.get,
        )

    @cached_property
    def operations(self) -> OperationsResourceWithRawResponse:
        return OperationsResourceWithRawResponse(self._discovery.operations)


class AsyncDiscoveryResourceWithRawResponse:
    def __init__(self, discovery: AsyncDiscoveryResource) -> None:
        self._discovery = discovery

        self.get = async_to_raw_response_wrapper(
            discovery.get,
        )

    @cached_property
    def operations(self) -> AsyncOperationsResourceWithRawResponse:
        return AsyncOperationsResourceWithRawResponse(self._discovery.operations)


class DiscoveryResourceWithStreamingResponse:
    def __init__(self, discovery: DiscoveryResource) -> None:
        self._discovery = discovery

        self.get = to_streamed_response_wrapper(
            discovery.get,
        )

    @cached_property
    def operations(self) -> OperationsResourceWithStreamingResponse:
        return OperationsResourceWithStreamingResponse(self._discovery.operations)


class AsyncDiscoveryResourceWithStreamingResponse:
    def __init__(self, discovery: AsyncDiscoveryResource) -> None:
        self._discovery = discovery

        self.get = async_to_streamed_response_wrapper(
            discovery.get,
        )

    @cached_property
    def operations(self) -> AsyncOperationsResourceWithStreamingResponse:
        return AsyncOperationsResourceWithStreamingResponse(self._discovery.operations)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.firewall import ua_rule_list_params, ua_rule_create_params, ua_rule_update_params
from ...types.firewall.ua_rule_get_response import UARuleGetResponse
from ...types.firewall.ua_rule_list_response import UARuleListResponse
from ...types.firewall.ua_rule_create_response import UARuleCreateResponse
from ...types.firewall.ua_rule_delete_response import UARuleDeleteResponse
from ...types.firewall.ua_rule_update_response import UARuleUpdateResponse

__all__ = ["UARulesResource", "AsyncUARulesResource"]


class UARulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UARulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UARulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UARulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UARulesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        configuration: ua_rule_create_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        description: str | Omit = omit,
        paused: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleCreateResponse:
        """
        Creates a new User Agent Blocking rule in a zone.

        Args:
          zone_id: Defines an identifier.

          mode: The action to apply to a matched request.

          description: An informative summary of the rule. This value is sanitized and any tags will be
              removed.

          paused: When true, indicates that the rule is currently paused.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/firewall/ua_rules", zone_id=zone_id),
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "description": description,
                    "paused": paused,
                },
                ua_rule_create_params.UARuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleCreateResponse], ResultWrapper[UARuleCreateResponse]),
        )

    def update(
        self,
        ua_rule_id: str,
        *,
        zone_id: str,
        configuration: ua_rule_update_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        description: str | Omit = omit,
        paused: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleUpdateResponse:
        """
        Updates an existing User Agent Blocking rule.

        Args:
          zone_id: Defines an identifier.

          ua_rule_id: The unique identifier of the User Agent Blocking rule.

          configuration: The rule configuration.

          mode: The action to apply to a matched request.

          description: An informative summary of the rule. This value is sanitized and any tags will be
              removed.

          paused: When true, indicates that the rule is currently paused.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not ua_rule_id:
            raise ValueError(f"Expected a non-empty value for `ua_rule_id` but received {ua_rule_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/firewall/ua_rules/{ua_rule_id}", zone_id=zone_id, ua_rule_id=ua_rule_id),
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "description": description,
                    "paused": paused,
                },
                ua_rule_update_params.UARuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleUpdateResponse], ResultWrapper[UARuleUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        description: str | Omit = omit,
        page: float | Omit = omit,
        paused: bool | Omit = omit,
        per_page: float | Omit = omit,
        user_agent: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[UARuleListResponse]:
        """Fetches User Agent Blocking rules in a zone.

        You can filter the results using
        several optional parameters.

        Args:
          zone_id: Defines an identifier.

          description: A string to search for in the description of existing rules.

          page: Page number of paginated results.

          paused: When true, indicates that the rule is currently paused.

          per_page: The maximum number of results per page. You can only set the value to `1` or to
              a multiple of 5 such as `5`, `10`, `15`, or `20`.

          user_agent: A string to search for in the user agent values of existing rules.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/ua_rules", zone_id=zone_id),
            page=SyncV4PagePaginationArray[UARuleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "description": description,
                        "page": page,
                        "paused": paused,
                        "per_page": per_page,
                        "user_agent": user_agent,
                    },
                    ua_rule_list_params.UARuleListParams,
                ),
            ),
            model=UARuleListResponse,
        )

    def delete(
        self,
        ua_rule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleDeleteResponse:
        """
        Deletes an existing User Agent Blocking rule.

        Args:
          zone_id: Defines an identifier.

          ua_rule_id: The unique identifier of the User Agent Blocking rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not ua_rule_id:
            raise ValueError(f"Expected a non-empty value for `ua_rule_id` but received {ua_rule_id!r}")
        return self._delete(
            path_template("/zones/{zone_id}/firewall/ua_rules/{ua_rule_id}", zone_id=zone_id, ua_rule_id=ua_rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleDeleteResponse], ResultWrapper[UARuleDeleteResponse]),
        )

    def get(
        self,
        ua_rule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleGetResponse:
        """
        Fetches the details of a User Agent Blocking rule.

        Args:
          zone_id: Defines an identifier.

          ua_rule_id: The unique identifier of the User Agent Blocking rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not ua_rule_id:
            raise ValueError(f"Expected a non-empty value for `ua_rule_id` but received {ua_rule_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/firewall/ua_rules/{ua_rule_id}", zone_id=zone_id, ua_rule_id=ua_rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleGetResponse], ResultWrapper[UARuleGetResponse]),
        )


class AsyncUARulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUARulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUARulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUARulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUARulesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        configuration: ua_rule_create_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        description: str | Omit = omit,
        paused: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleCreateResponse:
        """
        Creates a new User Agent Blocking rule in a zone.

        Args:
          zone_id: Defines an identifier.

          mode: The action to apply to a matched request.

          description: An informative summary of the rule. This value is sanitized and any tags will be
              removed.

          paused: When true, indicates that the rule is currently paused.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/firewall/ua_rules", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "description": description,
                    "paused": paused,
                },
                ua_rule_create_params.UARuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleCreateResponse], ResultWrapper[UARuleCreateResponse]),
        )

    async def update(
        self,
        ua_rule_id: str,
        *,
        zone_id: str,
        configuration: ua_rule_update_params.Configuration,
        mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"],
        description: str | Omit = omit,
        paused: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleUpdateResponse:
        """
        Updates an existing User Agent Blocking rule.

        Args:
          zone_id: Defines an identifier.

          ua_rule_id: The unique identifier of the User Agent Blocking rule.

          configuration: The rule configuration.

          mode: The action to apply to a matched request.

          description: An informative summary of the rule. This value is sanitized and any tags will be
              removed.

          paused: When true, indicates that the rule is currently paused.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not ua_rule_id:
            raise ValueError(f"Expected a non-empty value for `ua_rule_id` but received {ua_rule_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/firewall/ua_rules/{ua_rule_id}", zone_id=zone_id, ua_rule_id=ua_rule_id),
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "mode": mode,
                    "description": description,
                    "paused": paused,
                },
                ua_rule_update_params.UARuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleUpdateResponse], ResultWrapper[UARuleUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        description: str | Omit = omit,
        page: float | Omit = omit,
        paused: bool | Omit = omit,
        per_page: float | Omit = omit,
        user_agent: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UARuleListResponse, AsyncV4PagePaginationArray[UARuleListResponse]]:
        """Fetches User Agent Blocking rules in a zone.

        You can filter the results using
        several optional parameters.

        Args:
          zone_id: Defines an identifier.

          description: A string to search for in the description of existing rules.

          page: Page number of paginated results.

          paused: When true, indicates that the rule is currently paused.

          per_page: The maximum number of results per page. You can only set the value to `1` or to
              a multiple of 5 such as `5`, `10`, `15`, or `20`.

          user_agent: A string to search for in the user agent values of existing rules.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/ua_rules", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[UARuleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "description": description,
                        "page": page,
                        "paused": paused,
                        "per_page": per_page,
                        "user_agent": user_agent,
                    },
                    ua_rule_list_params.UARuleListParams,
                ),
            ),
            model=UARuleListResponse,
        )

    async def delete(
        self,
        ua_rule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleDeleteResponse:
        """
        Deletes an existing User Agent Blocking rule.

        Args:
          zone_id: Defines an identifier.

          ua_rule_id: The unique identifier of the User Agent Blocking rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not ua_rule_id:
            raise ValueError(f"Expected a non-empty value for `ua_rule_id` but received {ua_rule_id!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/firewall/ua_rules/{ua_rule_id}", zone_id=zone_id, ua_rule_id=ua_rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleDeleteResponse], ResultWrapper[UARuleDeleteResponse]),
        )

    async def get(
        self,
        ua_rule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UARuleGetResponse:
        """
        Fetches the details of a User Agent Blocking rule.

        Args:
          zone_id: Defines an identifier.

          ua_rule_id: The unique identifier of the User Agent Blocking rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not ua_rule_id:
            raise ValueError(f"Expected a non-empty value for `ua_rule_id` but received {ua_rule_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/firewall/ua_rules/{ua_rule_id}", zone_id=zone_id, ua_rule_id=ua_rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[UARuleGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[UARuleGetResponse], ResultWrapper[UARuleGetResponse]),
        )


class UARulesResourceWithRawResponse:
    def __init__(self, ua_rules: UARulesResource) -> None:
        self._ua_rules = ua_rules

        self.create = to_raw_response_wrapper(
            ua_rules.create,
        )
        self.update = to_raw_response_wrapper(
            ua_rules.update,
        )
        self.list = to_raw_response_wrapper(
            ua_rules.list,
        )
        self.delete = to_raw_response_wrapper(
            ua_rules.delete,
        )
        self.get = to_raw_response_wrapper(
            ua_rules.get,
        )


class AsyncUARulesResourceWithRawResponse:
    def __init__(self, ua_rules: AsyncUARulesResource) -> None:
        self._ua_rules = ua_rules

        self.create = async_to_raw_response_wrapper(
            ua_rules.create,
        )
        self.update = async_to_raw_response_wrapper(
            ua_rules.update,
        )
        self.list = async_to_raw_response_wrapper(
            ua_rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            ua_rules.delete,
        )
        self.get = async_to_raw_response_wrapper(
            ua_rules.get,
        )


class UARulesResourceWithStreamingResponse:
    def __init__(self, ua_rules: UARulesResource) -> None:
        self._ua_rules = ua_rules

        self.create = to_streamed_response_wrapper(
            ua_rules.create,
        )
        self.update = to_streamed_response_wrapper(
            ua_rules.update,
        )
        self.list = to_streamed_response_wrapper(
            ua_rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            ua_rules.delete,
        )
        self.get = to_streamed_response_wrapper(
            ua_rules.get,
        )


class AsyncUARulesResourceWithStreamingResponse:
    def __init__(self, ua_rules: AsyncUARulesResource) -> None:
        self._ua_rules = ua_rules

        self.create = async_to_streamed_response_wrapper(
            ua_rules.create,
        )
        self.update = async_to_streamed_response_wrapper(
            ua_rules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ua_rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ua_rules.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            ua_rules.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PolicyListResponse"]


class PolicyListResponse(BaseModel):
    id: str
    """Identifier"""

    action: Literal["allow", "log", "add_reporting_directives"]
    """The action to take if the expression matches"""

    description: str
    """A description for the policy"""

    enabled: bool
    """Whether the policy is enabled"""

    expression: str
    """
    The expression which must match for the policy to be applied, using the
    Cloudflare Firewall rule expression syntax
    """

    value: str
    """The policy which will be applied"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List
from typing_extensions import Literal, Required, TypedDict

from .phase import Phase
from ..._types import SequenceNotStr
from .logging_param import LoggingParam

__all__ = ["SkipRuleParam", "ActionParameters", "ExposedCredentialCheck", "Ratelimit"]


class ActionParameters(TypedDict, total=False):
    """The parameters configuring the rule's action."""

    phase: Literal["current"]
    """A phase to skip the execution of.

    This option is only compatible with the products option.
    """

    phases: List[Phase]
    """A list of phases to skip the execution of.

    This option is incompatible with the rulesets option.
    """

    products: List[Literal["bic", "hot", "rateLimit", "securityLevel", "uaBlock", "waf", "zoneLockdown"]]
    """A list of legacy security products to skip the execution of."""

    rules: Dict[str, SequenceNotStr[str]]
    """
    A mapping of ruleset IDs to a list of rule IDs in that ruleset to skip the
    execution of. This option is incompatible with the ruleset option.
    """

    ruleset: Literal["current"]
    """A ruleset to skip the execution of.

    This option is incompatible with the rulesets option.
    """

    rulesets: SequenceNotStr[str]
    """A list of ruleset IDs to skip the execution of.

    This option is incompatible with the ruleset and phases options.
    """


class ExposedCredentialCheck(TypedDict, total=False):
    """Configuration for exposed credential checking."""

    password_expression: Required[str]
    """An expression that selects the password used in the credentials check."""

    username_expression: Required[str]
    """An expression that selects the user ID used in the credentials check."""


class Ratelimit(TypedDict, total=False):
    """An object configuring the rule's rate limit behavior."""

    characteristics: Required[SequenceNotStr[str]]
    """
    Characteristics of the request on which the rate limit counter will be
    incremented.
    """

    period: Required[int]
    """Period in seconds over which the counter is being incremented."""

    counting_expression: str
    """An expression that defines when the rate limit counter should be incremented.

    It defaults to the same as the rule's expression.
    """

    mitigation_timeout: int
    """
    Period of time in seconds after which the action will be disabled following its
    first execution.
    """

    requests_per_period: int
    """
    The threshold of requests per period after which the action will be executed for
    the first time.
    """

    requests_to_origin: bool
    """Whether counting is only performed when an origin is reached."""

    score_per_period: int
    """
    The score threshold per period for which the action will be executed the first
    time.
    """

    score_response_header_name: str
    """
    A response header name provided by the origin, which contains the score to
    increment rate limit counter with.
    """


class SkipRuleParam(TypedDict, total=False):
    id: str
    """The unique ID of the rule."""

    action: Literal["skip"]
    """The action to perform when the rule matches."""

    action_parameters: ActionParameters
    """The parameters configuring the rule's action."""

    description: str
    """An informative description of the rule."""

    enabled: bool
    """Whether the rule should be executed."""

    exposed_credential_check: ExposedCredentialCheck
    """Configuration for exposed credential checking."""

    expression: str
    """The expression defining which traffic will match the rule."""

    logging: LoggingParam
    """An object configuring the rule's logging behavior."""

    ratelimit: Ratelimit
    """An object configuring the rule's rate limit behavior."""

    ref: str
    """The reference of the rule (the rule's ID by default)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["RuleUpdateParams", "Rule"]


class RuleUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Use this field to specify the unique ID of the zone."""

    rules: Required[Iterable[Rule]]
    """Lists snippet rules."""


class Rule(TypedDict, total=False):
    """Define a snippet rule."""

    expression: Required[str]
    """Define the expression that determines which traffic matches the rule."""

    snippet_name: Required[str]
    """Identify the snippet."""

    description: str
    """Provide an informative description of the rule."""

    enabled: bool
    """Indicate whether to execute the rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.zero_trust import connectivity_setting_edit_params
from ...types.zero_trust.connectivity_setting_get_response import ConnectivitySettingGetResponse
from ...types.zero_trust.connectivity_setting_edit_response import ConnectivitySettingEditResponse

__all__ = ["ConnectivitySettingsResource", "AsyncConnectivitySettingsResource"]


class ConnectivitySettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConnectivitySettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ConnectivitySettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConnectivitySettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ConnectivitySettingsResourceWithStreamingResponse(self)

    def edit(
        self,
        *,
        account_id: str,
        icmp_proxy_enabled: bool | Omit = omit,
        offramp_warp_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectivitySettingEditResponse:
        """
        Updates the Zero Trust Connectivity Settings for the given account.

        Args:
          account_id: Cloudflare account ID

          icmp_proxy_enabled: A flag to enable the ICMP proxy for the account network.

          offramp_warp_enabled: A flag to enable WARP to WARP traffic.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/zerotrust/connectivity_settings", account_id=account_id),
            body=maybe_transform(
                {
                    "icmp_proxy_enabled": icmp_proxy_enabled,
                    "offramp_warp_enabled": offramp_warp_enabled,
                },
                connectivity_setting_edit_params.ConnectivitySettingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectivitySettingEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectivitySettingEditResponse], ResultWrapper[ConnectivitySettingEditResponse]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectivitySettingGetResponse:
        """
        Gets the Zero Trust Connectivity Settings for the given account.

        Args:
          account_id: Cloudflare account ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/zerotrust/connectivity_settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectivitySettingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectivitySettingGetResponse], ResultWrapper[ConnectivitySettingGetResponse]),
        )


class AsyncConnectivitySettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConnectivitySettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConnectivitySettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConnectivitySettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncConnectivitySettingsResourceWithStreamingResponse(self)

    async def edit(
        self,
        *,
        account_id: str,
        icmp_proxy_enabled: bool | Omit = omit,
        offramp_warp_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectivitySettingEditResponse:
        """
        Updates the Zero Trust Connectivity Settings for the given account.

        Args:
          account_id: Cloudflare account ID

          icmp_proxy_enabled: A flag to enable the ICMP proxy for the account network.

          offramp_warp_enabled: A flag to enable WARP to WARP traffic.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/zerotrust/connectivity_settings", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "icmp_proxy_enabled": icmp_proxy_enabled,
                    "offramp_warp_enabled": offramp_warp_enabled,
                },
                connectivity_setting_edit_params.ConnectivitySettingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectivitySettingEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectivitySettingEditResponse], ResultWrapper[ConnectivitySettingEditResponse]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectivitySettingGetResponse:
        """
        Gets the Zero Trust Connectivity Settings for the given account.

        Args:
          account_id: Cloudflare account ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/zerotrust/connectivity_settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectivitySettingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectivitySettingGetResponse], ResultWrapper[ConnectivitySettingGetResponse]),
        )


class ConnectivitySettingsResourceWithRawResponse:
    def __init__(self, connectivity_settings: ConnectivitySettingsResource) -> None:
        self._connectivity_settings = connectivity_settings

        self.edit = to_raw_response_wrapper(
            connectivity_settings.edit,
        )
        self.get = to_raw_response_wrapper(
            connectivity_settings.get,
        )


class AsyncConnectivitySettingsResourceWithRawResponse:
    def __init__(self, connectivity_settings: AsyncConnectivitySettingsResource) -> None:
        self._connectivity_settings = connectivity_settings

        self.edit = async_to_raw_response_wrapper(
            connectivity_settings.edit,
        )
        self.get = async_to_raw_response_wrapper(
            connectivity_settings.get,
        )


class ConnectivitySettingsResourceWithStreamingResponse:
    def __init__(self, connectivity_settings: ConnectivitySettingsResource) -> None:
        self._connectivity_settings = connectivity_settings

        self.edit = to_streamed_response_wrapper(
            connectivity_settings.edit,
        )
        self.get = to_streamed_response_wrapper(
            connectivity_settings.get,
        )


class AsyncConnectivitySettingsResourceWithStreamingResponse:
    def __init__(self, connectivity_settings: AsyncConnectivitySettingsResource) -> None:
        self._connectivity_settings = connectivity_settings

        self.edit = async_to_streamed_response_wrapper(
            connectivity_settings.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            connectivity_settings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .policy import Policy as Policy
from .history import History as History
from .mechanism import Mechanism as Mechanism
from .policy_filter import PolicyFilter as PolicyFilter
from .mechanism_param import MechanismParam as MechanismParam
from .history_list_params import HistoryListParams as HistoryListParams
from .policy_filter_param import PolicyFilterParam as PolicyFilterParam
from .policy_create_params import PolicyCreateParams as PolicyCreateParams
from .policy_update_params import PolicyUpdateParams as PolicyUpdateParams
from .silence_get_response import SilenceGetResponse as SilenceGetResponse
from .silence_create_params import SilenceCreateParams as SilenceCreateParams
from .silence_list_response import SilenceListResponse as SilenceListResponse
from .silence_update_params import SilenceUpdateParams as SilenceUpdateParams
from .policy_create_response import PolicyCreateResponse as PolicyCreateResponse
from .policy_delete_response import PolicyDeleteResponse as PolicyDeleteResponse
from .policy_update_response import PolicyUpdateResponse as PolicyUpdateResponse
from .silence_create_response import SilenceCreateResponse as SilenceCreateResponse
from .silence_delete_response import SilenceDeleteResponse as SilenceDeleteResponse
from .silence_update_response import SilenceUpdateResponse as SilenceUpdateResponse
from .available_alert_list_response import AvailableAlertListResponse as AvailableAlertListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from .scan_result import ScanResult

__all__ = ["ResultGetResponse"]


class ResultGetResponse(BaseModel):
    one_one_one_one: List[ScanResult] = FieldInfo(alias="1.1.1.1")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.workers import account_setting_update_params
from ...types.workers.account_setting_get_response import AccountSettingGetResponse
from ...types.workers.account_setting_update_response import AccountSettingUpdateResponse

__all__ = ["AccountSettingsResource", "AsyncAccountSettingsResource"]


class AccountSettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AccountSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AccountSettingsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        default_usage_model: str | Omit = omit,
        green_compute: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountSettingUpdateResponse:
        """
        Creates Worker account settings for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/workers/account-settings", account_id=account_id),
            body=maybe_transform(
                {
                    "default_usage_model": default_usage_model,
                    "green_compute": green_compute,
                },
                account_setting_update_params.AccountSettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccountSettingUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccountSettingUpdateResponse], ResultWrapper[AccountSettingUpdateResponse]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountSettingGetResponse:
        """
        Fetches Worker account settings for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/workers/account-settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccountSettingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccountSettingGetResponse], ResultWrapper[AccountSettingGetResponse]),
        )


class AsyncAccountSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAccountSettingsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        default_usage_model: str | Omit = omit,
        green_compute: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountSettingUpdateResponse:
        """
        Creates Worker account settings for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/workers/account-settings", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "default_usage_model": default_usage_model,
                    "green_compute": green_compute,
                },
                account_setting_update_params.AccountSettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccountSettingUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccountSettingUpdateResponse], ResultWrapper[AccountSettingUpdateResponse]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountSettingGetResponse:
        """
        Fetches Worker account settings for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/workers/account-settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AccountSettingGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AccountSettingGetResponse], ResultWrapper[AccountSettingGetResponse]),
        )


class AccountSettingsResourceWithRawResponse:
    def __init__(self, account_settings: AccountSettingsResource) -> None:
        self._account_settings = account_settings

        self.update = to_raw_response_wrapper(
            account_settings.update,
        )
        self.get = to_raw_response_wrapper(
            account_settings.get,
        )


class AsyncAccountSettingsResourceWithRawResponse:
    def __init__(self, account_settings: AsyncAccountSettingsResource) -> None:
        self._account_settings = account_settings

        self.update = async_to_raw_response_wrapper(
            account_settings.update,
        )
        self.get = async_to_raw_response_wrapper(
            account_settings.get,
        )


class AccountSettingsResourceWithStreamingResponse:
    def __init__(self, account_settings: AccountSettingsResource) -> None:
        self._account_settings = account_settings

        self.update = to_streamed_response_wrapper(
            account_settings.update,
        )
        self.get = to_streamed_response_wrapper(
            account_settings.get,
        )


class AsyncAccountSettingsResourceWithStreamingResponse:
    def __init__(self, account_settings: AsyncAccountSettingsResource) -> None:
        self._account_settings = account_settings

        self.update = async_to_streamed_response_wrapper(
            account_settings.update,
        )
        self.get = async_to_streamed_response_wrapper(
            account_settings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["FirewallInput"]


class FirewallInput(BaseModel):
    enabled: bool
    """Enabled."""

    operating_system: Literal["windows", "mac"]
    """Operating System."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["LabelBulkDeleteResponse", "Label"]


class Label(BaseModel):
    created_at: datetime

    description: str
    """The description of the label"""

    last_updated: datetime

    metadata: object
    """Metadata for the label"""

    name: str
    """The name of the label"""

    source: Literal["user", "managed"]
    """
    - `user` - label is owned by the user
    - `managed` - label is owned by cloudflare
    """


class LabelBulkDeleteResponse(BaseModel):
    endpoint: str
    """
    The endpoint which can contain path parameter templates in curly braces, each
    will be replaced from left to right with {varN}, starting with {var1}, during
    insertion. This will further be Cloudflare-normalized upon insertion. See:
    https://developers.cloudflare.com/rules/normalization/how-it-works/.
    """

    host: str
    """RFC3986-compliant host."""

    last_updated: datetime

    method: Literal["GET", "POST", "HEAD", "OPTIONS", "PUT", "DELETE", "CONNECT", "PATCH", "TRACE"]
    """The HTTP method used to access the endpoint."""

    operation_id: str
    """UUID."""

    labels: Optional[List[Label]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AdvertisementStatusEditParams"]


class AdvertisementStatusEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of a Cloudflare account."""

    advertised: Required[bool]
    """Advertisement status of the prefix.

    If `true`, the BGP route for the prefix is advertised to the Internet. If
    `false`, the BGP route is withdrawn.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["ValidateOriginParams"]


class ValidateOriginParams(TypedDict, total=False):
    logpull_options: Required[Optional[str]]
    """This field is deprecated.

    Use `output_options` instead. Configuration string. It specifies things like
    requested fields and timestamp formats. If migrating from the logpull api, copy
    the url (full url or just the query string) of your call here, and logpush will
    keep on making this call for you, setting start and end times appropriately.
    """

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["IPRule", "IP"]


class IP(BaseModel):
    ip: str
    """An IPv4 or IPv6 CIDR block."""


class IPRule(BaseModel):
    """Matches an IP address block."""

    ip: IP



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["SecretCreateParams", "Body"]


class SecretCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account Identifier"""

    body: Required[Iterable[Body]]


class Body(TypedDict, total=False):
    name: Required[str]
    """The name of the secret"""

    scopes: Required[SequenceNotStr[str]]
    """The list of services that can use this secret."""

    value: Required[str]
    """The value of the secret.

    Maximum 64 KiB (65,536 bytes). Note that this is 'write only' - no API response
    will provide this value, it is only used to create/modify secrets.
    """

    comment: str
    """Freeform text describing the secret"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .job_list_params import JobListParams as JobListParams
from .job_get_response import JobGetResponse as JobGetResponse
from .job_create_params import JobCreateParams as JobCreateParams
from .job_list_response import JobListResponse as JobListResponse
from .job_abort_response import JobAbortResponse as JobAbortResponse
from .job_pause_response import JobPauseResponse as JobPauseResponse
from .job_create_response import JobCreateResponse as JobCreateResponse
from .job_resume_response import JobResumeResponse as JobResumeResponse
from .job_progress_response import JobProgressResponse as JobProgressResponse
from .job_abort_all_response import JobAbortAllResponse as JobAbortAllResponse
from .connectivity_precheck_source_params import ConnectivityPrecheckSourceParams as ConnectivityPrecheckSourceParams
from .connectivity_precheck_target_params import ConnectivityPrecheckTargetParams as ConnectivityPrecheckTargetParams
from .connectivity_precheck_source_response import (
    ConnectivityPrecheckSourceResponse as ConnectivityPrecheckSourceResponse,
)
from .connectivity_precheck_target_response import (
    ConnectivityPrecheckTargetResponse as ConnectivityPrecheckTargetResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PagePreviewResponse"]


class PagePreviewResponse(BaseModel):
    preview_url: Optional[str] = None
    """URL where the custom waiting room page can temporarily be previewed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.r2.super_slurper import job_list_params, job_create_params
from .....types.r2.super_slurper.job_get_response import JobGetResponse
from .....types.r2.super_slurper.job_list_response import JobListResponse
from .....types.r2.super_slurper.job_abort_response import JobAbortResponse
from .....types.r2.super_slurper.job_pause_response import JobPauseResponse
from .....types.r2.super_slurper.job_create_response import JobCreateResponse
from .....types.r2.super_slurper.job_resume_response import JobResumeResponse
from .....types.r2.super_slurper.job_progress_response import JobProgressResponse
from .....types.r2.super_slurper.job_abort_all_response import JobAbortAllResponse

__all__ = ["JobsResource", "AsyncJobsResource"]


class JobsResource(SyncAPIResource):
    @cached_property
    def logs(self) -> LogsResource:
        return LogsResource(self._client)

    @cached_property
    def with_raw_response(self) -> JobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return JobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return JobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        overwrite: bool | Omit = omit,
        source: job_create_params.Source | Omit = omit,
        target: job_create_params.Target | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[JobCreateResponse]:
        """
        Creates a new R2 Super Slurper migration job to transfer objects from a source
        bucket (e.g. S3, GCS, R2) to R2.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/slurper/jobs", account_id=account_id),
            body=maybe_transform(
                {
                    "overwrite": overwrite,
                    "source": source,
                    "target": target,
                },
                job_create_params.JobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[JobCreateResponse]], ResultWrapper[JobCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[JobListResponse]:
        """
        Lists all R2 Super Slurper migration jobs for the account with their status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/slurper/jobs", account_id=account_id),
            page=SyncSinglePage[JobListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=JobListResponse,
        )

    def abort(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Cancels a specific R2 Super Slurper migration job.

        Any objects in the middle of
        a transfer will finish, but no new objects will start transferring.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}/abort", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobAbortResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def abort_all(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Cancels all running R2 Super Slurper migration jobs for the account.

        Any objects
        in the middle of a transfer will finish, but no new objects will start
        transferring.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/slurper/jobs/abortAll", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobAbortAllResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def get(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[JobGetResponse]:
        """
        Retrieves detailed status and configuration for a specific R2 Super Slurper
        migration job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[JobGetResponse]], ResultWrapper[JobGetResponse]),
        )

    def pause(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Pauses a running R2 Super Slurper migration job.

        The job can be resumed later to
        continue transferring.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}/pause", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobPauseResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def progress(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[JobProgressResponse]:
        """
        Retrieves current progress metrics for an R2 Super Slurper migration job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/slurper/jobs/{job_id}/progress", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobProgressResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[JobProgressResponse]], ResultWrapper[JobProgressResponse]),
        )

    def resume(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Resumes a paused R2 Super Slurper migration job, continuing the transfer from
        where it stopped.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}/resume", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobResumeResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class AsyncJobsResource(AsyncAPIResource):
    @cached_property
    def logs(self) -> AsyncLogsResource:
        return AsyncLogsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        overwrite: bool | Omit = omit,
        source: job_create_params.Source | Omit = omit,
        target: job_create_params.Target | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[JobCreateResponse]:
        """
        Creates a new R2 Super Slurper migration job to transfer objects from a source
        bucket (e.g. S3, GCS, R2) to R2.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/slurper/jobs", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "overwrite": overwrite,
                    "source": source,
                    "target": target,
                },
                job_create_params.JobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[JobCreateResponse]], ResultWrapper[JobCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[JobListResponse, AsyncSinglePage[JobListResponse]]:
        """
        Lists all R2 Super Slurper migration jobs for the account with their status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/slurper/jobs", account_id=account_id),
            page=AsyncSinglePage[JobListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=JobListResponse,
        )

    async def abort(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Cancels a specific R2 Super Slurper migration job.

        Any objects in the middle of
        a transfer will finish, but no new objects will start transferring.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}/abort", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobAbortResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    async def abort_all(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Cancels all running R2 Super Slurper migration jobs for the account.

        Any objects
        in the middle of a transfer will finish, but no new objects will start
        transferring.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/slurper/jobs/abortAll", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobAbortAllResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    async def get(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[JobGetResponse]:
        """
        Retrieves detailed status and configuration for a specific R2 Super Slurper
        migration job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[JobGetResponse]], ResultWrapper[JobGetResponse]),
        )

    async def pause(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Pauses a running R2 Super Slurper migration job.

        The job can be resumed later to
        continue transferring.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}/pause", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobPauseResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    async def progress(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[JobProgressResponse]:
        """
        Retrieves current progress metrics for an R2 Super Slurper migration job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/slurper/jobs/{job_id}/progress", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobProgressResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[JobProgressResponse]], ResultWrapper[JobProgressResponse]),
        )

    async def resume(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Resumes a paused R2 Super Slurper migration job, continuing the transfer from
        where it stopped.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/slurper/jobs/{job_id}/resume", account_id=account_id, job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[JobResumeResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class JobsResourceWithRawResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.create = to_raw_response_wrapper(
            jobs.create,
        )
        self.list = to_raw_response_wrapper(
            jobs.list,
        )
        self.abort = to_raw_response_wrapper(
            jobs.abort,
        )
        self.abort_all = to_raw_response_wrapper(
            jobs.abort_all,
        )
        self.get = to_raw_response_wrapper(
            jobs.get,
        )
        self.pause = to_raw_response_wrapper(
            jobs.pause,
        )
        self.progress = to_raw_response_wrapper(
            jobs.progress,
        )
        self.resume = to_raw_response_wrapper(
            jobs.resume,
        )

    @cached_property
    def logs(self) -> LogsResourceWithRawResponse:
        return LogsResourceWithRawResponse(self._jobs.logs)


class AsyncJobsResourceWithRawResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.create = async_to_raw_response_wrapper(
            jobs.create,
        )
        self.list = async_to_raw_response_wrapper(
            jobs.list,
        )
        self.abort = async_to_raw_response_wrapper(
            jobs.abort,
        )
        self.abort_all = async_to_raw_response_wrapper(
            jobs.abort_all,
        )
        self.get = async_to_raw_response_wrapper(
            jobs.get,
        )
        self.pause = async_to_raw_response_wrapper(
            jobs.pause,
        )
        self.progress = async_to_raw_response_wrapper(
            jobs.progress,
        )
        self.resume = async_to_raw_response_wrapper(
            jobs.resume,
        )

    @cached_property
    def logs(self) -> AsyncLogsResourceWithRawResponse:
        return AsyncLogsResourceWithRawResponse(self._jobs.logs)


class JobsResourceWithStreamingResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.create = to_streamed_response_wrapper(
            jobs.create,
        )
        self.list = to_streamed_response_wrapper(
            jobs.list,
        )
        self.abort = to_streamed_response_wrapper(
            jobs.abort,
        )
        self.abort_all = to_streamed_response_wrapper(
            jobs.abort_all,
        )
        self.get = to_streamed_response_wrapper(
            jobs.get,
        )
        self.pause = to_streamed_response_wrapper(
            jobs.pause,
        )
        self.progress = to_streamed_response_wrapper(
            jobs.progress,
        )
        self.resume = to_streamed_response_wrapper(
            jobs.resume,
        )

    @cached_property
    def logs(self) -> LogsResourceWithStreamingResponse:
        return LogsResourceWithStreamingResponse(self._jobs.logs)


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.create = async_to_streamed_response_wrapper(
            jobs.create,
        )
        self.list = async_to_streamed_response_wrapper(
            jobs.list,
        )
        self.abort = async_to_streamed_response_wrapper(
            jobs.abort,
        )
        self.abort_all = async_to_streamed_response_wrapper(
            jobs.abort_all,
        )
        self.get = async_to_streamed_response_wrapper(
            jobs.get,
        )
        self.pause = async_to_streamed_response_wrapper(
            jobs.pause,
        )
        self.progress = async_to_streamed_response_wrapper(
            jobs.progress,
        )
        self.resume = async_to_streamed_response_wrapper(
            jobs.resume,
        )

    @cached_property
    def logs(self) -> AsyncLogsResourceWithStreamingResponse:
        return AsyncLogsResourceWithStreamingResponse(self._jobs.logs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = [
    "ConfigurationGetResponse",
    "Config",
    "ConfigIngress",
    "ConfigIngressOriginRequest",
    "ConfigIngressOriginRequestAccess",
    "ConfigOriginRequest",
    "ConfigOriginRequestAccess",
]


class ConfigIngressOriginRequestAccess(BaseModel):
    """
    For all L7 requests to this hostname, cloudflared will validate each request's Cf-Access-Jwt-Assertion request header.
    """

    aud_tag: List[str] = FieldInfo(alias="audTag")
    """Access applications that are allowed to reach this hostname for this Tunnel.

    Audience tags can be identified in the dashboard or via the List Access policies
    API.
    """

    team_name: str = FieldInfo(alias="teamName")

    required: Optional[bool] = None
    """Deny traffic that has not fulfilled Access authorization."""


class ConfigIngressOriginRequest(BaseModel):
    """
    Configuration parameters for the public hostname specific connection settings between cloudflared and origin server.
    """

    access: Optional[ConfigIngressOriginRequestAccess] = None
    """
    For all L7 requests to this hostname, cloudflared will validate each request's
    Cf-Access-Jwt-Assertion request header.
    """

    ca_pool: Optional[str] = FieldInfo(alias="caPool", default=None)
    """Path to the certificate authority (CA) for the certificate of your origin.

    This option should be used only if your certificate is not signed by Cloudflare.
    """

    connect_timeout: Optional[int] = FieldInfo(alias="connectTimeout", default=None)
    """Timeout for establishing a new TCP connection to your origin server.

    This excludes the time taken to establish TLS, which is controlled by
    tlsTimeout.
    """

    disable_chunked_encoding: Optional[bool] = FieldInfo(alias="disableChunkedEncoding", default=None)
    """Disables chunked transfer encoding. Useful if you are running a WSGI server."""

    http2_origin: Optional[bool] = FieldInfo(alias="http2Origin", default=None)
    """Attempt to connect to origin using HTTP2. Origin must be configured as https."""

    http_host_header: Optional[str] = FieldInfo(alias="httpHostHeader", default=None)
    """Sets the HTTP Host header on requests sent to the local service."""

    keep_alive_connections: Optional[int] = FieldInfo(alias="keepAliveConnections", default=None)
    """Maximum number of idle keepalive connections between Tunnel and your origin.

    This does not restrict the total number of concurrent connections.
    """

    keep_alive_timeout: Optional[int] = FieldInfo(alias="keepAliveTimeout", default=None)
    """Timeout after which an idle keepalive connection can be discarded."""

    match_sn_ito_host: Optional[bool] = FieldInfo(alias="matchSNItoHost", default=None)
    """Auto configure the Hostname on the origin server certificate."""

    no_happy_eyeballs: Optional[bool] = FieldInfo(alias="noHappyEyeballs", default=None)
    """
    Disable the “happy eyeballs” algorithm for IPv4/IPv6 fallback if your local
    network has misconfigured one of the protocols.
    """

    no_tls_verify: Optional[bool] = FieldInfo(alias="noTLSVerify", default=None)
    """Disables TLS verification of the certificate presented by your origin.

    Will allow any certificate from the origin to be accepted.
    """

    origin_server_name: Optional[str] = FieldInfo(alias="originServerName", default=None)
    """Hostname that cloudflared should expect from your origin server certificate."""

    proxy_type: Optional[str] = FieldInfo(alias="proxyType", default=None)
    """
    cloudflared starts a proxy server to translate HTTP traffic into TCP when
    proxying, for example, SSH or RDP. This configures what type of proxy will be
    started. Valid options are: "" for the regular proxy and "socks" for a SOCKS5
    proxy.
    """

    tcp_keep_alive: Optional[int] = FieldInfo(alias="tcpKeepAlive", default=None)
    """
    The timeout after which a TCP keepalive packet is sent on a connection between
    Tunnel and the origin server.
    """

    tls_timeout: Optional[int] = FieldInfo(alias="tlsTimeout", default=None)
    """
    Timeout for completing a TLS handshake to your origin server, if you have chosen
    to connect Tunnel to an HTTPS server.
    """


class ConfigIngress(BaseModel):
    """Public hostname"""

    hostname: str
    """Public hostname for this service."""

    service: str
    """Protocol and address of destination server.

    Supported protocols: http://, https://, unix://, tcp://, ssh://, rdp://,
    unix+tls://, smb://. Alternatively can return a HTTP status code
    http_status:[code] e.g. 'http_status:404'.
    """

    origin_request: Optional[ConfigIngressOriginRequest] = FieldInfo(alias="originRequest", default=None)
    """
    Configuration parameters for the public hostname specific connection settings
    between cloudflared and origin server.
    """

    path: Optional[str] = None
    """Requests with this path route to this public hostname."""


class ConfigOriginRequestAccess(BaseModel):
    """
    For all L7 requests to this hostname, cloudflared will validate each request's Cf-Access-Jwt-Assertion request header.
    """

    aud_tag: List[str] = FieldInfo(alias="audTag")
    """Access applications that are allowed to reach this hostname for this Tunnel.

    Audience tags can be identified in the dashboard or via the List Access policies
    API.
    """

    team_name: str = FieldInfo(alias="teamName")

    required: Optional[bool] = None
    """Deny traffic that has not fulfilled Access authorization."""


class ConfigOriginRequest(BaseModel):
    """
    Configuration parameters for the public hostname specific connection settings between cloudflared and origin server.
    """

    access: Optional[ConfigOriginRequestAccess] = None
    """
    For all L7 requests to this hostname, cloudflared will validate each request's
    Cf-Access-Jwt-Assertion request header.
    """

    ca_pool: Optional[str] = FieldInfo(alias="caPool", default=None)
    """Path to the certificate authority (CA) for the certificate of your origin.

    This option should be used only if your certificate is not signed by Cloudflare.
    """

    connect_timeout: Optional[int] = FieldInfo(alias="connectTimeout", default=None)
    """Timeout for establishing a new TCP connection to your origin server.

    This excludes the time taken to establish TLS, which is controlled by
    tlsTimeout.
    """

    disable_chunked_encoding: Optional[bool] = FieldInfo(alias="disableChunkedEncoding", default=None)
    """Disables chunked transfer encoding. Useful if you are running a WSGI server."""

    http2_origin: Optional[bool] = FieldInfo(alias="http2Origin", default=None)
    """Attempt to connect to origin using HTTP2. Origin must be configured as https."""

    http_host_header: Optional[str] = FieldInfo(alias="httpHostHeader", default=None)
    """Sets the HTTP Host header on requests sent to the local service."""

    keep_alive_connections: Optional[int] = FieldInfo(alias="keepAliveConnections", default=None)
    """Maximum number of idle keepalive connections between Tunnel and your origin.

    This does not restrict the total number of concurrent connections.
    """

    keep_alive_timeout: Optional[int] = FieldInfo(alias="keepAliveTimeout", default=None)
    """Timeout after which an idle keepalive connection can be discarded."""

    match_sn_ito_host: Optional[bool] = FieldInfo(alias="matchSNItoHost", default=None)
    """Auto configure the Hostname on the origin server certificate."""

    no_happy_eyeballs: Optional[bool] = FieldInfo(alias="noHappyEyeballs", default=None)
    """
    Disable the “happy eyeballs” algorithm for IPv4/IPv6 fallback if your local
    network has misconfigured one of the protocols.
    """

    no_tls_verify: Optional[bool] = FieldInfo(alias="noTLSVerify", default=None)
    """Disables TLS verification of the certificate presented by your origin.

    Will allow any certificate from the origin to be accepted.
    """

    origin_server_name: Optional[str] = FieldInfo(alias="originServerName", default=None)
    """Hostname that cloudflared should expect from your origin server certificate."""

    proxy_type: Optional[str] = FieldInfo(alias="proxyType", default=None)
    """
    cloudflared starts a proxy server to translate HTTP traffic into TCP when
    proxying, for example, SSH or RDP. This configures what type of proxy will be
    started. Valid options are: "" for the regular proxy and "socks" for a SOCKS5
    proxy.
    """

    tcp_keep_alive: Optional[int] = FieldInfo(alias="tcpKeepAlive", default=None)
    """
    The timeout after which a TCP keepalive packet is sent on a connection between
    Tunnel and the origin server.
    """

    tls_timeout: Optional[int] = FieldInfo(alias="tlsTimeout", default=None)
    """
    Timeout for completing a TLS handshake to your origin server, if you have chosen
    to connect Tunnel to an HTTPS server.
    """


class Config(BaseModel):
    """The tunnel configuration and ingress rules."""

    ingress: Optional[List[ConfigIngress]] = None
    """List of public hostname definitions.

    At least one ingress rule needs to be defined for the tunnel.
    """

    origin_request: Optional[ConfigOriginRequest] = FieldInfo(alias="originRequest", default=None)
    """
    Configuration parameters for the public hostname specific connection settings
    between cloudflared and origin server.
    """


class ConfigurationGetResponse(BaseModel):
    """Cloudflare Tunnel configuration"""

    account_id: Optional[str] = None
    """Identifier."""

    config: Optional[Config] = None
    """The tunnel configuration and ingress rules."""

    created_at: Optional[datetime] = None

    source: Optional[Literal["local", "cloudflare"]] = None
    """Indicates if this is a locally or remotely configured tunnel.

    If `local`, manage the tunnel using a YAML file on the origin machine. If
    `cloudflare`, manage the tunnel's configuration on the Zero Trust dashboard.
    """

    tunnel_id: Optional[str] = None
    """UUID of the tunnel."""

    version: Optional[int] = None
    """The version of the Tunnel Configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo
from ...r2.buckets.provider import Provider

__all__ = [
    "InstanceCreateParams",
    "CustomMetadata",
    "IndexMethod",
    "IndexingOptions",
    "Metadata",
    "PublicEndpointParams",
    "PublicEndpointParamsChatCompletionsEndpoint",
    "PublicEndpointParamsMcp",
    "PublicEndpointParamsRateLimit",
    "PublicEndpointParamsSearchEndpoint",
    "RetrievalOptions",
    "RetrievalOptionsBoostBy",
    "SourceParams",
    "SourceParamsWebCrawler",
    "SourceParamsWebCrawlerCrawlOptions",
    "SourceParamsWebCrawlerParseOptions",
    "SourceParamsWebCrawlerParseOptionsContentSelector",
    "SourceParamsWebCrawlerStoreOptions",
]


class InstanceCreateParams(TypedDict, total=False):
    account_id: Required[str]

    id: Required[str]
    """AI Search instance ID. Lowercase alphanumeric, hyphens, and underscores."""

    ai_gateway_id: Optional[str]

    aisearch_model: Annotated[
        Optional[
            Literal[
                "@cf/meta/llama-3.3-70b-instruct-fp8-fast",
                "@cf/zai-org/glm-4.7-flash",
                "@cf/meta/llama-3.1-8b-instruct-fast",
                "@cf/meta/llama-3.1-8b-instruct-fp8",
                "@cf/meta/llama-4-scout-17b-16e-instruct",
                "@cf/qwen/qwen3-30b-a3b-fp8",
                "@cf/deepseek-ai/deepseek-r1-distill-qwen-32b",
                "@cf/moonshotai/kimi-k2-instruct",
                "@cf/google/gemma-3-12b-it",
                "@cf/google/gemma-4-26b-a4b-it",
                "@cf/moonshotai/kimi-k2.5",
                "anthropic/claude-3-7-sonnet",
                "anthropic/claude-sonnet-4",
                "anthropic/claude-opus-4",
                "anthropic/claude-3-5-haiku",
                "cerebras/qwen-3-235b-a22b-instruct",
                "cerebras/qwen-3-235b-a22b-thinking",
                "cerebras/llama-3.3-70b",
                "cerebras/llama-4-maverick-17b-128e-instruct",
                "cerebras/llama-4-scout-17b-16e-instruct",
                "cerebras/gpt-oss-120b",
                "google-ai-studio/gemini-2.5-flash",
                "google-ai-studio/gemini-2.5-pro",
                "grok/grok-4",
                "groq/llama-3.3-70b-versatile",
                "groq/llama-3.1-8b-instant",
                "openai/gpt-5",
                "openai/gpt-5-mini",
                "openai/gpt-5-nano",
                "",
            ]
        ],
        PropertyInfo(alias="ai_search_model"),
    ]

    cache: bool

    cache_threshold: Literal["super_strict_match", "close_enough", "flexible_friend", "anything_goes"]

    cache_ttl: Literal[600, 1800, 3600, 7200, 21600, 43200, 86400, 172800, 259200, 518400]
    """Cache entry TTL in seconds.

    Allowed values: 600 (10min), 1800 (30min), 3600 (1h), 7200 (2h), 21600 (6h),
    43200 (12h), 86400 (24h), 172800 (48h), 259200 (72h), 518400 (6d).
    """

    chunk: bool

    chunk_overlap: int

    chunk_size: int

    custom_metadata: Iterable[CustomMetadata]

    embedding_model: Optional[
        Literal[
            "@cf/qwen/qwen3-embedding-0.6b",
            "@cf/baai/bge-m3",
            "@cf/baai/bge-large-en-v1.5",
            "@cf/google/embeddinggemma-300m",
            "google-ai-studio/gemini-embedding-001",
            "google-ai-studio/gemini-embedding-2-preview",
            "openai/text-embedding-3-small",
            "openai/text-embedding-3-large",
            "",
        ]
    ]

    fusion_method: Literal["max", "rrf"]

    hybrid_search_enabled: bool
    """Deprecated — use index_method instead."""

    index_method: IndexMethod
    """Controls which storage backends are used during indexing.

    Defaults to vector-only.
    """

    indexing_options: Optional[IndexingOptions]

    max_num_results: int

    metadata: Metadata

    public_endpoint_params: PublicEndpointParams

    reranking: bool

    reranking_model: Optional[Literal["@cf/baai/bge-reranker-base", ""]]

    retrieval_options: Optional[RetrievalOptions]

    rewrite_model: Optional[
        Literal[
            "@cf/meta/llama-3.3-70b-instruct-fp8-fast",
            "@cf/zai-org/glm-4.7-flash",
            "@cf/meta/llama-3.1-8b-instruct-fast",
            "@cf/meta/llama-3.1-8b-instruct-fp8",
            "@cf/meta/llama-4-scout-17b-16e-instruct",
            "@cf/qwen/qwen3-30b-a3b-fp8",
            "@cf/deepseek-ai/deepseek-r1-distill-qwen-32b",
            "@cf/moonshotai/kimi-k2-instruct",
            "@cf/google/gemma-3-12b-it",
            "@cf/google/gemma-4-26b-a4b-it",
            "@cf/moonshotai/kimi-k2.5",
            "anthropic/claude-3-7-sonnet",
            "anthropic/claude-sonnet-4",
            "anthropic/claude-opus-4",
            "anthropic/claude-3-5-haiku",
            "cerebras/qwen-3-235b-a22b-instruct",
            "cerebras/qwen-3-235b-a22b-thinking",
            "cerebras/llama-3.3-70b",
            "cerebras/llama-4-maverick-17b-128e-instruct",
            "cerebras/llama-4-scout-17b-16e-instruct",
            "cerebras/gpt-oss-120b",
            "google-ai-studio/gemini-2.5-flash",
            "google-ai-studio/gemini-2.5-pro",
            "grok/grok-4",
            "groq/llama-3.3-70b-versatile",
            "groq/llama-3.1-8b-instant",
            "openai/gpt-5",
            "openai/gpt-5-mini",
            "openai/gpt-5-nano",
            "",
        ]
    ]

    rewrite_query: bool

    score_threshold: float

    source: Optional[str]

    source_params: Optional[SourceParams]

    sync_interval: Literal[900, 1800, 3600, 7200, 14400, 21600, 43200, 86400]
    """Interval between automatic syncs, in seconds.

    Allowed values: 900 (15min), 1800 (30min), 3600 (1h), 7200 (2h), 14400 (4h),
    21600 (6h), 43200 (12h), 86400 (24h).
    """

    token_id: str

    type: Optional[Literal["r2", "web-crawler"]]


class CustomMetadata(TypedDict, total=False):
    data_type: Required[Literal["text", "number", "boolean", "datetime"]]

    field_name: Required[str]


class IndexMethod(TypedDict, total=False):
    """Controls which storage backends are used during indexing.

    Defaults to vector-only.
    """

    keyword: Required[bool]
    """Enable keyword (BM25) storage backend."""

    vector: Required[bool]
    """Enable vector (embedding) storage backend."""


class IndexingOptions(TypedDict, total=False):
    keyword_tokenizer: Literal["porter", "trigram"]
    """Tokenizer used for keyword search indexing.

    porter provides word-level tokenization with Porter stemming (good for natural
    language queries). trigram enables character-level substring matching (good for
    partial matches, code, identifiers). Changing this triggers a full re-index.
    Defaults to porter.
    """


class Metadata(TypedDict, total=False):
    created_from_aisearch_wizard: bool

    worker_domain: str


class PublicEndpointParamsChatCompletionsEndpoint(TypedDict, total=False):
    disabled: bool
    """Disable chat completions endpoint for this public endpoint"""


class PublicEndpointParamsMcp(TypedDict, total=False):
    description: str

    disabled: bool
    """Disable MCP endpoint for this public endpoint"""


class PublicEndpointParamsRateLimit(TypedDict, total=False):
    period_ms: int

    requests: int

    technique: Literal["fixed", "sliding"]


class PublicEndpointParamsSearchEndpoint(TypedDict, total=False):
    disabled: bool
    """Disable search endpoint for this public endpoint"""


class PublicEndpointParams(TypedDict, total=False):
    authorized_hosts: SequenceNotStr[str]

    chat_completions_endpoint: PublicEndpointParamsChatCompletionsEndpoint

    enabled: bool

    mcp: PublicEndpointParamsMcp

    rate_limit: PublicEndpointParamsRateLimit

    search_endpoint: PublicEndpointParamsSearchEndpoint


class RetrievalOptionsBoostBy(TypedDict, total=False):
    field: Required[str]
    """Metadata field name to boost by.

    Use 'timestamp' for document freshness, or any custom_metadata field. Numeric
    and datetime fields support asc/desc directions; text/boolean fields support
    exists/not_exists.
    """

    direction: Literal["asc", "desc", "exists", "not_exists"]
    """Boost direction.

    'desc' = higher values rank higher (e.g. newer timestamps). 'asc' = lower values
    rank higher. 'exists' = boost chunks that have the field. 'not_exists' = boost
    chunks that lack the field. Optional ��� defaults to 'asc' for numeric/datetime
    fields, 'exists' for text/boolean fields.
    """


class RetrievalOptions(TypedDict, total=False):
    boost_by: Iterable[RetrievalOptionsBoostBy]
    """Metadata fields to boost search results by.

    Each entry specifies a metadata field and an optional direction. Direction
    defaults to 'asc' for numeric fields and 'exists' for text/boolean fields.
    Fields must match 'timestamp' or a defined custom_metadata field.
    """

    keyword_match_mode: Literal["and", "or"]
    """Controls which documents are candidates for BM25 scoring.

    'and' restricts candidates to documents containing all query terms; 'or'
    includes any document containing at least one term, ranked by BM25 relevance.
    Defaults to 'and'.
    """


class SourceParamsWebCrawlerCrawlOptions(TypedDict, total=False):
    depth: float

    include_external_links: bool

    include_subdomains: bool

    max_age: float

    source: Literal["all", "sitemaps", "links"]


class SourceParamsWebCrawlerParseOptionsContentSelector(TypedDict, total=False):
    path: Required[str]
    """Glob pattern to match against the page URL path.

    Uses standard glob syntax: \\** matches within a segment, \\**\\** crosses
    directories.
    """

    selector: Required[str]
    """CSS selector to extract content from pages matching the path pattern.

    Supports standard CSS selectors including class, ID, element, and attribute
    selectors.
    """


class SourceParamsWebCrawlerParseOptions(TypedDict, total=False):
    content_selector: Iterable[SourceParamsWebCrawlerParseOptionsContentSelector]
    """
    List of path-to-selector mappings for extracting specific content from crawled
    pages. Each entry pairs a URL glob pattern with a CSS selector. The first
    matching path wins. Only the matched HTML fragment is stored and indexed.
    """

    include_headers: Dict[str, str]

    include_images: bool

    specific_sitemaps: SequenceNotStr[str]
    """List of specific sitemap URLs to use for crawling.

    Only valid when parse_type is 'sitemap'.
    """

    use_browser_rendering: bool


class SourceParamsWebCrawlerStoreOptions(TypedDict, total=False):
    storage_id: Required[str]

    r2_jurisdiction: str

    storage_type: Provider


class SourceParamsWebCrawler(TypedDict, total=False):
    crawl_options: SourceParamsWebCrawlerCrawlOptions

    parse_options: SourceParamsWebCrawlerParseOptions

    parse_type: Literal["sitemap", "feed-rss", "crawl"]

    store_options: SourceParamsWebCrawlerStoreOptions


class SourceParams(TypedDict, total=False):
    exclude_items: SequenceNotStr[str]
    """List of path patterns to exclude.

    Uses micromatch glob syntax: \\** matches within a path segment, ** matches across
    path segments (e.g., /admin/** matches /admin/users and
    /admin/settings/advanced)
    """

    include_items: SequenceNotStr[str]
    """List of path patterns to include.

    Uses micromatch glob syntax: \\** matches within a path segment, ** matches across
    path segments (e.g., /blog/** matches /blog/post and /blog/2024/post)
    """

    prefix: str

    r2_jurisdiction: str

    web_crawler: SourceParamsWebCrawler
